# 使用
## 1、将代码上传到cdb，并编译：
    go build -mod vendor -o subscribe main/main.go
    
## 2、启动消费：
    ./subscribe --brokers=xxx --topic=xxx --group=xxx --user=xxx --password=xxx --trans2sql=true


# 启动参数说明
| 参数      |       说明                      |
| ----------| -------------------------------|
| brokerlist| Kafka 的内网访问地址（域名+端口号） |
| topic     | Kafka topic 名称                |
| group     | Kafka 消费组名称                 |
| user      | Kafka 消费组账号名               |
| password  | Kafka 消费组密码                 |
| trans2sql | 是否将消费数据转换为 SQL 语句打印   |


# Record中字段说明

| 字段名称               |     类型       |含义                      |
| ---------------       | --------      | ----------               |
| id                    |   long        | 全局递增Id                |
| version               |   int         | 协议版本，当前版本为1       |
| messageType           |   String      | 消息类型，枚举值："INSERT", "UPDATE", "DELETE", "DDL", "BEGIN", "COMMIT", "HEARTBEAT", "CHECKPOINT", "STATEMENT", "INIT_DDL", "INIT_INSERT" |
| fileName              |   String      | 当前record所在的binlog文件名 |
| position              |   String      | 当前record的在binlog中结束的偏移量，格式为 End_log_pos@binlog文件编号。比如，当前record位于文件mysql-bin.000004中，结束偏移量为2196，则其值为"2196@4" |
| safePosition          |   String      | 当前事务在binlog中开始的偏移量，格式同上   |
| timestamp             |   long        | 写入binlog的时间，unix时间戳，秒级        |
| gtid                  |   String      | 当前的gtid，如：c7c98333-6006-11ed-bfc9-b8cef6e1a231:9   |
| transactionId         |   String      | 事务Id，只有commit事件才会生成事务id                 |
| serverId              |   String      | 源库serverId，查看源库server_id参考 SHOW VARIABLES LIKE 'server_id'  |
| threadId              |   String      | 提交当前事务的会话id，参考 SHOW processlist;         |
| sourceType            |   String      | 源库类型，当前版本只有 MySQL                         |
| sourceVersion         |   String      | 源库版本，查看源库版本参考 SELECT version()           |
| schemaName            |   String      | 库名                   |
| tableName             |   String      | 表名                   |
| objectName            |   String      | 格式为：库名.表名        |
| columns               |   List\<Field> | 表中各列的定义          |
| oldColumns            |   List\<String>| dml执行前该行的数据，如果是insert消息，该数组为null    |
| newColumns            |   List\<String>| dml执行后该行的数据，如果是delete消息，该数组为null    |
| sql                   |   String      | ddl的sql语句            |
| executionTime         |   int         | ddl执行时长，单位：秒     |
| heartbeatTimestamp    |   long        | 心跳消息的时间戳，秒级。只有heartbeat消息有意义   |
| syncedGtid            |   String      | 已解析gtid集合，格式形如：c7c98333-6006-11ed-bfc9-b8cef6e1a231:1-13 |
| fakeGtid              |   boolean     | 是否为构造的假gtid。如未开启gtid_mode，则dts会构造一个gtid             |
| pkNames               |   List\<String>| 主键字段，只有dml消息可能有值                  | 
| readerTimestamp       |   long        | dts处理这条数据的时间，unix时间戳，毫秒级         |
| tags                  |   Map\<String,String>| QueryEvent中的status_vars，详细参考：https://dev.mysql.com/doc/dev/mysql-server/latest/classbinary__log_1_1Query__event.html       |
| total                 |   int         | 如果消息分片，记录分片总数。当前版本(version=1)无意义，预留扩展      |
| index                 |   int         | 如果消息分片，记录当前分片的索引。当前版本(version=1)无意义，预留扩展 |

## mysql列属性

+ name 列名
+ dataTypeNumber 是binlog中记录的数据类型。取值详见：https://dev.mysql.com/doc/internals/en/com-query-response.html
+ isKey 是否主键
+ originalType ddl中定义的类型

## mysql数据类型转换逻辑

在json协议中，将mysql类型全部转换成了字符串：
    + 对于varchar等字符串类型，全部转成了UTF8编码；
    + 对于数值类型，全部转成了与值相同的字符串，如 "3.0" ；
    + 对于时间类型，格式为：yyyy-dd-mm hh:MM:ss.milli；
    + 对于时间戳类型，输出为毫秒数；
    + 对于binary、blob等二进制类型，输出为与16进制值相同的字符串，如 "0xfff"；
    + 如果源库的**binlog_row_image**变量设置为**MINIMAL**，binlog不会记录未修改列，此时输出null；
    + 如果表中数据为null，此时输出空字符串。注意，这种情况下无法区分表中数据为null还是原本就是空字符串。