package json

const (
	HEARTBEAT   = "HEARTBEAT"
	CHECKPOINT  = "CHECKPOINT"
	BEGIN       = "BEGIN"
	COMMIT      = "COMMIT"
	ROLLBACK    = "ROLLBACK"
	DDL         = "DDL"
	INSERT      = "INSERT"
	DELETE      = "DELETE"
	UPDATE      = "UPDATE"
	STATEMENT   = "STATEMENT"
	INIT_DDL    = "INIT_DDL"
	INIT_INSERT = "INIT_INSERT"
)

// PlatRecord json格式的输出
type PlatRecord struct {
	// unique id of this record in the whole stream
	Id int64 `json:"id"`
	// json version
	Version int32 `json:"version"`

	MessageType string `json:"messageType"`
	// binlog file name
	FileName string `json:"fileName"`
	// record log timestamp
	Timestamp int64 `json:"timestamp"`

	Gtid string `json:"gtid"`
	// end_log_pos of last event (exclude commit) in transaction
	Position string `json:"position"`
	// end_log_pos of begin event in transaction
	SafePosition string `json:"safePosition"`
	// transaction id
	TransactionId string `json:"transactionId"`

	ServerId string `json:"serverId"`

	ThreadId string `json:"threadId"`
	// source database type
	SourceType string `json:"sourceType"`
	// source database version
	SourceVersion string `json:"sourceVersion"`
	// name of source schema if exist
	SchemaName string `json:"schemaName"`
	// name of source table if exist
	TableName string `json:"tableName"`
	// schemaName.tableName
	ObjectName string `json:"objectName"`

	Columns []*Field `json:"columns"`
	// values of a row after modify. if values' size is over 20M, they will be discarded
	// type for this filed should be either []byte or []string
	OldColumns interface{} `json:"oldColumns"`
	// values of a row after modify. if values' size is over 20M, they will be discarded
	// type for this filed should be either []byte or []string
	NewColumns interface{} `json:"newColumns"`
	// ddl sql if this is a ddl event
	Sql string `json:"sql"`
	// execution time of ddl sql
	ExecutionTime int32 `json:"executionTime"`
	// timestamp of heartbeat send by master if this is a heartbeat record
	HeartbeatTimestamp int64 `json:"heartbeatTimestamp"`

	SyncedGtid string `json:"syncedGtid"`
	//
	FakeGtid bool `json:"fakeGtid"`
	// primary keys
	PkNames []string `json:"pkNames"`
	// a millisecond timestamp. Not implemented at current version
	ReaderTimestamp int64 `json:"readerTimestamp"`
	// tags to identify properties of this record
	Tags map[string]string `json:"tags"`
	// Not implemented at current version
	Index int32 `json:"index"`
	// Not implemented at current version
	Total int32 `json:"total"`
}

type Field struct {
	// column names for table
	Name string `json:"name"`

	DataTypeNumber int32 `json:"dataTypeNumber"`

	IsKey bool `json:"isKey"`
	// datatype of this column in source db
	OriginalType string `json:"originalType"`
}
